import logging

from solace import SOLACE_LOGGING_CONFIG
from solace.messaging.messaging_service import MessagingService

LOGGER_NAME = 'pypubsub'


def init_logging(stream_level="WARNING", file_logging=False, c_log_level=None, c_log_file=None):
    level = stream_level.upper()
    set_python_solace_log_level(level)

    if c_log_level is not None:
        MessagingService.set_core_messaging_log_level(c_log_level, c_log_file)

    # create formatter to add to the handlers
    formatter = logging.Formatter('%(asctime)s <%(levelname)s> %(name)s: %(message)s')

    logger = logging.getLogger(LOGGER_NAME)
    logger.setLevel(logging.DEBUG)
    # Log everything to the file
    if file_logging:
        fh = logging.FileHandler('debug.log', mode='w')
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        logger.addHandler(fh)

    # Log warn and above to the terminal
    ch = logging.StreamHandler()
    ch.setLevel(level)
    ch.setFormatter(formatter)
    logger.addHandler(ch)


def set_python_solace_log_level(level: str):
    # get solace loggers
    loggers = SOLACE_LOGGING_CONFIG['loggers']
    # update log level
    for logr in loggers:
        loggers.get(logr)['level'] = level
    # set logger config change
    logging.config.dictConfig(SOLACE_LOGGING_CONFIG)


def get_logger():
    return logging.getLogger(LOGGER_NAME)
