from sdkperf.config.endpoint_access_type import EndpointAccessType
from sdkperf.config.subscriber_destinations_type import SubscriberDestinationsType


class EpConfigProperties:
    def __init__(self):
        self.ep_names = list()
        self.subscriptions = list()
        self.selectors = list()
        self.is_adding = False
        self.want_topic_unsubscribe = False
        self.no_local = False
        self.client_name = ""
        self.type = SubscriberDestinationsType.NULL
        self.num_temp_endpoints = 0
        self.respect_TTL = False
        self.transacted_session_name = ""
        self.transaction_size = 0
        self.discard_notify_sender = -1
        self.is_topic = False
        self.access_type = EndpointAccessType.ACCESSTYPE_API_DEFAULT
        self.max_msg_size = -1
        self.quota = -1
        self.permission = ""
        self.max_msg_redelivery = -1
        self.want_provision = False
        self.want_cut_through_persistence = False
        self.distribute_topics = False
        self.want_replay = False
        self.want_replay_from_date = ""
        self.want_replay_from_msg_id = ""
        self.reconnect_tries = -1
        self.reconnect_retry_interval_in_msecs = 3000
        self.settlement_patterns = list()
        self.required_settlements = list()

        self.want_active_flow_indication = False
        self.want_message_priority_order_checking = None

    @classmethod
    def create_default(cls):
        return cls()

    @classmethod
    def create_for_subscription_add(cls, subscriptions: list, client_name: str, sub_type: SubscriberDestinationsType):
        ret = cls()
        ret.subscriptions = subscriptions
        ret.is_adding = True
        ret.client_name = client_name
        ret.type = sub_type
        return ret

    @classmethod
    def create_for_subscription_remove(cls, subscriptions: list, client_name: str,
                                       sub_type: SubscriberDestinationsType):
        ret = cls()
        ret.subscriptions = subscriptions
        ret.is_adding = False
        ret.client_name = client_name
        ret.type = sub_type
        return ret

    @classmethod
    def create_for_queue_add(cls, ep_names: list, selectors: list):
        ret = cls()
        ret.ep_names = ep_names
        ret.is_adding = True
        ret.selectors = selectors
        return ret

    @classmethod
    def create_for_queue_remove(cls, ep_names: list):
        ret = cls()
        ret.ep_names = ep_names
        ret.is_adding = False
        return ret

    @classmethod
    def create_for_ep_provision(cls,
                                ep_names: list,
                                is_topic: bool,
                                is_adding: bool,
                                access_type: EndpointAccessType.ACCESSTYPE_API_DEFAULT,
                                max_msg_size: int,
                                quota: int,
                                permission: str,
                                respect_ttl: int,
                                max_msg_redelivery: int,
                                want_active_flow_indication: bool):
        ret = cls()
        ret.ep_names = ep_names
        ret.is_topic = is_topic
        ret.is_adding = is_adding
        ret.access_type = access_type
        ret.max_msg_size = max_msg_size
        ret.quota = quota
        ret.permission = permission
        ret.respect_TTL = respect_ttl
        ret.max_msg_redelivery = max_msg_redelivery
        ret.want_active_flow_indication = want_active_flow_indication
        ret.want_provision = True
        return ret

    def __str__(self):
        buf = ""
        buf += f"EP Names Size           : {len(self.ep_names)}\n"
        buf += f"Subscriptions Size      : {len(self.subscriptions)}\n"
        buf += f"Selectors Size          : {len(self.selectors)}\n"
        buf += f"Is Adding               : {self.is_adding}\n"
        buf += f"Want Topic Unsub        : {self.want_topic_unsubscribe}\n"
        buf += f"No Local                : {self.no_local}\n"
        buf += f"Client Name             : {self.client_name}\n"
        buf += f"Type                    : {self.type}\n"
        buf += f"Num Temp EPs            : {self.num_temp_endpoints}\n"
        buf += f"Respect TTL             : {self.respect_TTL}\n"
        buf += f"Trans Sess Name         : {self.transacted_session_name}\n"
        buf += f"Trans Size              : {self.transaction_size}\n"
        buf += f"Discard Notify Sender   : {self.discard_notify_sender}\n"
        buf += f"Is Topic                : {self.is_topic}\n"
        buf += f"Access Type             : {self.access_type}\n"
        buf += f"Max Message Size        : {self.max_msg_size}\n"
        buf += f"Quota                   : {self.quota}\n"
        buf += f"Permission              : {self.permission}\n"
        buf += f"Max Msg Redelivery      : {self.max_msg_redelivery}\n"
        buf += f"Want Provision          : {self.want_provision}\n"
        buf += f"Cut-through             : {self.want_cut_through_persistence}\n"
        buf += f"Distribute Topics       : {self.distribute_topics}\n"
        buf += f"Want replay             : {self.want_replay}\n"
        buf += f"Want replay from date   : {self.want_replay_from_date}\n"
        buf += f"Want replay from msg id : {self.want_replay_from_msg_id}\n"
        buf += f"Reconnect Tries         : {self.reconnect_tries}\n"
        buf += f"Reconnect Retry Interval: {self.reconnect_retry_interval_in_msecs}\n"
        buf += f"Want Active Flow Indication: {self.want_active_flow_indication}\n"
        buf += f"Settlement Patterns     : {len(self.settlement_patterns)}\n"
        buf += f"Required Settlements    : {len(self.required_settlements)}\n"
