import binascii

from solace.messaging.config.solace_properties.message_properties import HTTP_CONTENT_ENCODING, HTTP_CONTENT_TYPE, \
    PERSISTENT_DMQ_ELIGIBLE, ELIDING_ELIGIBLE, APPLICATION_MESSAGE_ID, PERSISTENT_TIME_TO_LIVE, CLASS_OF_SERVICE
from solace.messaging.publisher.outbound_message import OutboundMessageBuilder

from pypubsubtools.core.runtime_properties import RuntimeProperties
from sdkperf.core.sdm_properties import SdmProperties
from sdkperf.core.tool_info import ToolData


class MessageRep:
    def __init__(self, builder: OutboundMessageBuilder, props, client_id, smf=None, sdt_prop: SdmProperties = None):
        self.msg_props = dict()
        self.client_id = client_id

        self.builder = builder
        self.smf = smf
        self.sdt_prop = sdt_prop

        self._set_up_builder(props)
        self._internal_msg = None
        self.get_message_obj(True)

    def _set_up_builder(self, props):
        if getattr(props, RuntimeProperties.WANT_PAYLOAD_CHECK, False) and self.smf is not None:
            # The crc32 algorithm will always return an unsigned integer in Python3+, but all the
            # other APIs use a signed int32. If the crc32 algorithm produces an integer which requires
            # more than 32 bits, we need to manually mimic overflow behaviour.
            crc_val = binascii.crc32(self.smf) & 0xffffffff
            crc_val = crc_val | (-(crc_val & 0x80000000))
            self.msg_props[ToolData.PROP_BIN_PAYLOAD_HASH] = crc_val
        if getattr(props, RuntimeProperties.PUB_STREAM_ID_OFFSET, -1) >= 0:
            self.msg_props[ToolData.PROP_STREAM_ID] = getattr(props, RuntimeProperties.PUB_STREAM_ID_OFFSET) + self.client_id
        if getattr(props, RuntimeProperties.HTTP_CONTENT_ENCODING, None) is not None:
            self.msg_props[HTTP_CONTENT_ENCODING] = getattr(props, RuntimeProperties.HTTP_CONTENT_ENCODING)
        if getattr(props, RuntimeProperties.HTTP_CONTENT_TYPE, None) is not None:
            self.msg_props[HTTP_CONTENT_TYPE] = getattr(props, RuntimeProperties.HTTP_CONTENT_TYPE)
        if getattr(props, RuntimeProperties.MSG_DMQ_ELIGIBLE, None) is not None:
            self.msg_props[PERSISTENT_DMQ_ELIGIBLE] = getattr(props, RuntimeProperties.MSG_DMQ_ELIGIBLE, True)
        if getattr(props, RuntimeProperties.MSG_ELIDING_ELIGIBLE, None) is not None:
            self.msg_props[ELIDING_ELIGIBLE] = getattr(props, RuntimeProperties.MSG_ELIDING_ELIGIBLE, False)
        if getattr(props, RuntimeProperties.MESSAGE_ID, "") != "":
            self.msg_props[APPLICATION_MESSAGE_ID] = int(getattr(props, RuntimeProperties.MESSAGE_ID))
        if int(getattr(props, RuntimeProperties.MSG_TTL, 0)) > 0:
            self.msg_props[PERSISTENT_TIME_TO_LIVE] = int(getattr(props, RuntimeProperties.MSG_TTL))
        if int(getattr(props, RuntimeProperties.PUBLISH_COS, -1)) >= 0:
            self.msg_props[CLASS_OF_SERVICE] = int(getattr(props, RuntimeProperties.PUBLISH_COS)) - 1

        if getattr(props, RuntimeProperties.CUSTOM_PROPERTIES_LIST, None) is not None:
            self.msg_props = {**self.msg_props, **getattr(props, RuntimeProperties.CUSTOM_PROPERTIES_LIST)}

        if len(self.msg_props) == 0:
            self.msg_props = None

    def get_message_obj(self, want_new=False):
        if self._internal_msg is None or want_new:
            if self.sdt_prop is not None:
                self._internal_msg = self.builder.build(self.sdt_prop.get_payload(), self.sdt_prop.get_properties())
            else:
                self._internal_msg = self.builder.build(self.smf, self.msg_props)

        return self._internal_msg
