from solace.messaging.receiver.inbound_message import InboundMessage

from pypubsubtools.utils.log import get_logger
from sdkperf.core.abstract_wrapped_message import AbstractWrappedMessage

logger = get_logger()


class PythonWrappedMessage(AbstractWrappedMessage):
    def __init__(self, message: InboundMessage, delivery_mode):
        self._msg = message
        self._delivery_mode = delivery_mode

    def get_message(self):
        return self._msg

    def has_xml(self):
        logger.error("XML Payload is not supported in Python API")
        return False

    def has_attachment(self):
        return self._msg.get_payload_as_bytes() is not None

    def has_topic(self):
        if self._msg is None:
            return False
        return self._msg.get_destination_name() is not None

    def get_destination_name(self):
        if self._msg is None:
            return None
        return self._msg.get_destination_name()

    def get_destination_type(self):
        logger.error("Get Destination Type is not supported in Python API")
        return None

    def get_delivery_mode(self):
        if self._delivery_mode is None:
            return None
        return self._delivery_mode

    def get_xml_bytes(self):
        logger.error("XML Payload is not supported in Python API")
        return None

    def get_attachment_bytes(self):
        if self._msg is None:
            return None
        return self._msg.get_payload_as_bytes()

    def dump(self):
        if self._msg is None:
            logger.error("Unable to print None message")
            return None
        return self._msg.__str__()

    def get_message_as_bytes(self):
        logger.error("Method get_message_as_bytes not supported for this message type")
        return bytearray(0)

    def get_discard_indication(self):
        if self._msg is None:
            logger.error("Unable to check DI on None message")
            return False
        return self._msg.get_message_discard_notification().has_broker_discard_indication()

    def get_priority(self):
        if self._msg is None:
            return None
        return self._msg.get_priority()
